﻿;========================================================================================================================
; キーボードヘルパー for PASORAMA（DF-X10001 64bit版） Version 1.0
; 2015/03/26 内山卓則
;
; PASORAMAをキーボードで操作しやすくするツールです。主な機能は4つあります。
;
; ■(1)ウィンドウ上部のボタンをショートカットキーで操作できます。
; ・Ctrl+1：「辞書選択」ボタンをクリック（表示される一覧から辞書を手動選択）
; ・Ctrl+2：「一括検索」ボタンをクリック
; ・Ctrl+3：「例文検索」ボタンをクリック
; ・Ctrl+4：「成句検索」ボタンをクリック
; ・Ctrl+5：「日本語例文」ボタンをクリック
; ・Ctrl+6：「全文表示」ボタンのオン/オフを切り替え
; ・F1　　：ウィンドウ上部のパネル全体の表示/非表示を切り替え
; ・F2　　：検索語句の入力エリアにフォーカスを移動（パネルが非表示のときは代わりの入力ボックスを表示）
;
; ■(2)Alt+英数字キーで、個別の辞書を直接選択できます。キーボードの行ごとに、次のようにショートカットキーを割り振ってあります。
; ・Alt+1～-（キーボードの1行目）：英英辞典（Oxford、Longman、Collins等）
; ・Alt+q～@（キーボードの2行目）：英和辞典（ランダムハウス、新英和大辞典、リーダーズ等）
; ・Alt+a～h（キーボードの3行目）：和英辞典（新和英大辞典、プログレッシブ等）
; ・Alt+z～n（キーボードの4行目）：国語辞典（精選版日国、広辞苑等）
;
; ■(3)↑/↓/PgUp/PgDnキーを押したときに上下に移動する対象を、Ctrlの有無によって指定できます。
; ・Ctrl+↑/↓/PgUp/PgDn：検索結果リスト（ウィンドウの左側）を移動
; ・↑/↓/PgUp/PgDn 　　：辞書本文（ウィンドウの右側）を移動
;
; ■(4)一括検索の結果一覧リストで、辞書の種類（英和/英英/和英/国語）単位で項目をジャンプできます。
; ・Ctrl+Shift+↓：次の種類（英和/英英/和英/国語）の先頭にジャンプ
; ・Ctrl+Shift+↑：前の種類（英和/英英/和英/国語）の先頭にジャンプ
; ※スクリプトを修正すれば、特定の辞書のみにジャンプするようにもできます。
;
; ショートカットキーを変えたい場合や、上記(4)の機能でジャンプする辞書を変えたい場合は、
; コメントに「★」と付いているあたりを修正してください（数カ所にあります）。
;========================================================================================================================

#SingleInstance force
#NoEnv
#Warn
CoordMode, Mouse, Window    ; マウスの座標はアクティブウィンドウ上の相対位置
SendMode Input              ; ※キー送信がうまく動作しない場合は、InputをEventに変えてみてください


; ======== 辞書選択のショートカットキーの定義 ========
; ★ キーを変えたい場合は、以下の各行の " " 囲みの部分を修正してください（ショートカットキーが不要な辞書は空文字列 "" でも大丈夫です）。
; ※ 「:」の右にある 1～34 の数値は変えないでください。
keylist :=  { "!1" : 18  ;Oxford Dictionary of English
            , "!2" : 19  ;Oxford Advanced Learner's Dictionary
            , "!3" : 20  ;Longman Dictionary of Contemporary English
            , "!4" : 21  ;Collins COBUILD
            , "!5" : 22  ;Oxford Thesaurus of English
            , "!6" : 23  ;Oxford Collocations Dictionary
            , "!7" : 24  ;Oxford Phrasal Verbs Dictioanry
            , "!8" : 25  ;Oxford IDIOMS Dictionary
            , "!9" : 26  ;Oxford Guide to British and American Culture
            , "!0" : 27  ;Britannica Concise Encyclopedia
            , "!-" : 28  ;Longman Pronunciation Dictionary

            , "!q" :  1  ;ランダムハウス英和大辞典
            , "!w" :  2  ;新英和大辞典
            , "!e" :  3  ;リーダーズ英和辞典
            , "!r" :  4  ;リーダーズ・プラス
            , "!t" :  5  ;ジーニアス英和大辞典
            , "!y" :  6  ;新編 英和活用大辞典
            , "!u" :  7  ;ロングマン英和辞典
            , "!i" :  8  ;ナノテクノロジー用語英和辞典
            , "!o" :  9  ;180万語対訳大辞典
            , "!p" : 10  ;人文社会37万語対訳大辞典
            , "!@" : 11  ;ビジネス技術実用英語大辞典-英和

            , "!a" : 15  ;新和英大辞典
            , "!s" : 16  ;プログレッシブ和英中辞典
            , "!d" : 12  ;ビジネス技術実用英語大辞典-和英
            , "!f" : 13  ;自然科学系和英大辞典
            , "!g" : 14  ;英文ビジネスeメール実例集
            , "!h" : 17  ;カタカナで引くスペリング辞典

            , "!z" : 29  ;精選版日本国語大辞典
            , "!x" : 30  ;広辞苑 第六版
            , "!c" : 31  ;角川類語新辞典
            , "!v" : 32  ;世界大百科事典
            , "!b" : 33  ;明鏡国語辞典
            , "!n" : 34} ;新漢語林


; ======== 辞書選択のショートカットキーのハンドラ登録処理 ========
Hotkey, IfWinActive, PASORAMA, 接続(&F)
    for hk_i, hk_v in keylist {
        if (hk_i != "") { 
            Hotkey, %hk_i%, DictChg
        }
    }
Hotkey, IfWinActive


; ======== 辞書選択以外のショートカットキーのハンドラ ========
; ★ キーを変えたい場合は、下のブロック内の該当する行で、「::」の左側を修正してください。
; ※ キーに対する処理の中で A_ThisHotkey 変数を使っている場合、その部分にも修正が必要かもしれません。
#IfWinActive, PASORAMA, 接続(&F)
    ; == 辞書選択～日本語例文の各ボタンを選択するショートカットキー ==
    ^1::ButtonPush("Button18") ; 辞書選択
    ^2::ButtonPush("Button19") ; 一括検索
    ^3::ButtonPush("Button20") ; 例文検索
    ^4::ButtonPush("Button21") ; 成句検索
    ^5::ButtonPush("Button22") ; 日本語例文

    ; == 全文表示のオン/オフを切り替えるショートカットキー ==
    ^6::ButtonPush("Button11")

    ; == 検索結果の一覧を上下に移動するキー（※ルーチンは各キーで共用） ==
    ; ※割り当てるキーを変える場合、ルーチン内の「SubStr(A_ThisHotkey,2)」の部分にも修正が必要かもしれません。
    ^Up::
    ^Down::
    ^PgUp::
    ^PgDn::
    ^NumpadUp::
    ^NumpadDown::
    ^NumpadPgUp::
    ^NumpadPgDn::
        ControlSend, AfxFrameOrView42su1, % "{" . SubStr(A_ThisHotkey,2) . "}"  ; 先頭の「^」を削除して結果一覧リストにSend
        return

    ; == 辞書本文を上下に移動するキー（※ルーチンは各キーで共用）==
    ; ※割り当てるキーを変える場合、ルーチン内の「%A_ThisHotkey%」の部分にも修正が必要かもしれません。
    Up::
    Down::
    PgUp::
    PgDn::
    NumpadUp::
    NumpadDown::
    NumpadPgUp::
    NumpadPgDn::
        ControlGetFocus, hk_c
        if (hk_c == "Button18") {
            Send, {%A_ThisHotkey%}                              ; 辞書のリストを選択するときのキーはそのままSend
        } else {
            ControlSend, AfxFrameOrView42su2, {%A_ThisHotkey%}  ; それ以外は辞書本文のエリアにSend
        }
        return

    ; == 一括検索の結果で次/前の辞書グループに移動 ==
    ^+Up::ListJumpToNextGrp(-1)
    ^+NumpadUp::ListJumpToNextGrp(-1)
    ^+Down::ListJumpToNextGrp(1)
    ^+NumpadDown::ListJumpToNextGrp(1)

    ; == パネルの表示/非表示を切り替えるショートカットキー ==
    F1::PanelToggle()

    ; == 検索語句の入力ボックスを表示するショートカットキー ==
    F2::InputBoxShow()
#IfWinActive


; ======== 辞書選択のショートカットキーのハンドラ ========
DictChg:
    ; パネルが非表示なら表示
    ; （※この後呼び出すButtonPushの中にも同じ処理があるが、ボタン押下後に引き続き辞書選択を実行させるために、ここで制御しておく）
    hk_v := PanelIsVisible()
    if (!hk_v) {
        PanelToggle()
    }

    ; 入力エリアの内容を消す（こうしないと、辞書選択と同時に検索が実行され、一瞬ハングしたみたいに見えるため）
    ControlFocus, Edit1
    Send, {Home}+{End}{Del}

    ; 辞書選択ボタンを押し、目的の辞書を選択
    ButtonPush("Button18")
    Send, {Home}
    if (keylist[A_ThisHotkey] >= 2) {
        Send, % "{Down " . (keylist[A_ThisHotkey] - 1) . "}"
    }
    Send, {Enter}
    Sleep, 200

    ; 入力エリアの内容を元に戻す（ここで検索が実行される）
    ControlFocus, Edit1
    Send, ^z

    ; 最初にパネルが非表示だった場合はそのように戻す
    if (!hk_v) {
        PanelToggle()
    }
    
    return


; ======== ボタンを押す関数 ========
ButtonPush(btn)
{
    ; パネルが非表示なら表示
    v := PanelIsVisible()
    if (!v) {
        PanelToggle()
    }

    ; ボタン押下処理
    ControlGetPos, x, y, w, h, %btn%, PASORAMA, 接続(&F)
    x := x + w/2
    y := y + h/2
    Click, %x%, %y%

    ; 最初にパネルが非表示だった場合はそのように戻す
    if (!v) {
        PanelToggle()
    }

    return
}


; ======== ボタン表示部のパネルの表示/非表示を判別する関数 ========
PanelIsVisible()
{
    ControlGet, v, Visible, , ReBarWindow321, PASORAMA, 接続(&F)
    return v
}


; ======== ボタン表示部のパネルの表示/非表示を切り替える関数 ========
PanelToggle()
{
    ; パネルの表示/非表示を反転
    if (PanelIsVisible()) {
        Control, Hide, , ReBarWindow321
        ControlGetText, mode, Static1
        title := "PASORAMA - " . mode
    } else {
        Control, Show, , ReBarWindow321
        title := "PASORAMA"
    }

    ; ウィンドウを再描画
    WinGet, state, MinMax 
    if (state == 1) {
        WinRestore
        WinMaximize
    } else {
        WinGetPos, , , w, h
        WinMove, , , , , % w - 1, % h - 1
        WinMove, , , , , % w, % h
    }

    ; タイトルを設定
    WinSetTitle, , , %title%
    return
}


; ======== 入力ボックスを表示する関数 ========
InputBoxShow()
{
    ; パネルを表示中なら、入力エリアを全選択にするのみで終了
    if (PanelIsVisible()) {
        ControlFocus, Edit1
        Send, {Home}+{End}
        return
    } 

    ; 入力ボックスの位置を決定
    WinGetPos, x, y, w, h
    x := x + w/2 - 192
    y := y + h/2 - 64

    ; 入力ボックスを表示
    InputBox, str, PASORAMA, 検索語句, , 384, 128, x, y

    ; 入力内容を正規の入力エリアに転送
    if (!ErrorLevel && str) {
        clipbkup := Clipboard
        Clipboard := str
        WinActivate, PASORAMA, 接続(&F)
        ControlFocus, Edit1
        Send, {Home}+{End}^v
        Sleep, 200
        Clipboard := clipbkup
    }

    return
}


; ======== 検索結果リストのカーソルを特定の位置まで移動する関数 ========
ListJumpToNextGrp(direction)
{
    ; ※パラメータはカーソルの移動方向（direction>0なら下方向、direction<=0なら上方向）

    ; 辞書のグループ分け（0はジャンプの対象外）（デフォルトでは、英和/英英/和英/国語という区分）
    ; ★ グループ分けを変えたい場合は、行の右端の数字を変えてください。
    ; ★ 特定の辞書のみにジャンプさせたい場合は、対象の辞書に1、2、3、…と1つずつ番号を振り、残りの辞書はすべて0にしてください
    ; ※ 「:」の左にある辞書名は変えないでください。
    static dictgrp := {"ランダムハウス英和大辞典（第２版）" : 1
        , "新英和大辞典 第６版" : 1
        , "リーダーズ英和辞典（第３版）" : 1
        , "リーダーズ・プラス" : 1
        , "ジーニアス英和大辞典 用例プラス" : 1
        , "新編英和活用大辞典" : 1
        , "ロングマン英和辞典" : 1
        , "ナノテクノロジー用語英和辞典" : 1
        , "180万語対訳大辞典　科学・医学・工学・農学・化学・ビジネス" : 1
        , "人文社会37万語対訳大辞典" : 1
        , "ビジネス技術実用英語大辞典Ｖ５　英和編" : 1
        , "Oxford Dictionary of English, Third Edition" : 2
        , "Oxford Advanced Learner’s Dictionary, Eighth Edition" : 2
        , "Longman Dictionary of Contemporary English, Fifth Edition" : 2
        , "Collins COBUILD Advanced Dictionary of English" : 2
        , "Oxford Thesaurus of English, Third Edition" : 2
        , "Oxford Collocations Dictionary for students of English, Second Edition" : 2
        , "Oxford Phrasal Verbs Dictionary for learners of English, Second Edition" : 2
        , "Oxford IDIOMS Dictionary for learners of English, Second Edition" : 2
        , "Oxford Guide to British and American Culture, Second Edition" : 2
        , "Britannica Concise Encyclopedia" : 2
        , "Longman Pronunciation Dictionary, Third Edition" : 0
        , "新和英大辞典 電子増補版" : 3
        , "プログレッシブ和英中辞典（第３版）" : 3
        , "ビジネス技術実用英語大辞典Ｖ５　和英編" : 3
        , "自然科学系和英大辞典　増補改訂新版（第４版）" : 3
        , "英文ビジネスeメール実例集　Ver. 2.0" : 3
        , "カタカナで引くスペリング辞典" : 3
        , "精選版日本国語大辞典" : 4
        , "広辞苑 第六版" : 4
        , "角川類語新辞典" : 4
        , "平凡社 改訂新版 世界大百科事典" : 4
        , "明鏡国語辞典" : 4
        , "新漢語林" : 4}

    ; 一括検索モードでないときは、少し大きめの幅で移動するのみとする
    ControlGetText, mode, Static1
    if (mode != "一括検索") {
        ControlSend, AfxFrameOrView42su1, % "{" . ((direction > 0) ? "Down" : "Up") . " 7}"
        return
    }

    ; 上方向の場合、最初に1つ上に行っておく（現在位置がグループの先頭だった場合に、前のグループの先頭まで飛ばしたいから）
    if (direction <= 0) {
        ControlSend, AfxFrameOrView42su1, {Up}
    }

    ; 次のグループまでリストを移動（ただし、無限ループ状態を防ぐために、同じ辞書またはグループが一定行に達した場合はそこまでとする）
    i := 0                          ; 全体のループカウンタ
    d := 0                          ; 同じ辞書が続いた回数のカウンタ
    StatusBarGetText, prevdict      ; 前の行の辞書名を覚えておくための変数
    key := (direction > 0) ? "{Down}" : "{Up}"
    Loop {
        ; 1つ位置を動かして辞書名を取得
        ControlSend, AfxFrameOrView42su1, %key%
        StatusBarGetText, dict

        ; 辞書グループの境界を越えたかチェック
        if (dictgrp[dict] != dictgrp[prevdict])  {
            if (direction > 0) && (dictgrp[dict] != 0) {
                ; 下に移動するときは、辞書グループの境界を越えた時点のグループが0でなければ終了
                break
            } else if (direction <= 0) && (dictgrp[prevdict] != 0) {
                ; 上に移動するときは、辞書グループの境界を越える直前のグループが0でなければ終了
                ControlSend, AfxFrameOrView42su1, {Down}    ; 最後に1つ下に戻る（境界を越える直前が目的地なので）
                break
            }
        }

        ; 辞書の反復回数を越えたかチェック
        if (dict != prevdict) {
            d := 0
        } else {
            d += 1
            if ((dict != "180万語対訳大辞典　科学・医学・工学・農学・化学・ビジネス") && (d >= 15)) || (d >= 30) {
                break
            } 
        }

        prevdict := dict
        i += 1
    } Until (i >= 60)

    return
}
